using Distributed
using ArgParse

@everywhere using JLD
@everywhere using StatsBase
@everywhere using SharedArrays



function userArgs()
    arglist = ArgParseSettings()
    @add_arg_table arglist begin
        "--cpus","-c"
            help = "CPU number for parallel computation"
            arg_type = Int
            default = 8
        "--iteration","-n"
              help = "Iteration times"
              arg_type = Int
              default = 100
         "--subsampling","-m"
              help = "Numbers of subgroups, subsampling for stablility selection, pleaase keep default for most users"
              arg_type = Int
              default=2
        "--percentcutoff","-p"
              help = "Ranking frequency percentage for cutoff (0-1)"
              arg_type = Float64
              default=0.95
        "-i"
            help = "Input file name, required"
            arg_type = String
            required = true
         "--clinic","-t"
                help = "Input time and status file name, required"
                arg_type = String
                required = true
        "-o"
            help = "Output file name, required"
            arg_type=String
            required =true
    end
    return parse_args(arglist)
end

##read in user input
pargs = userArgs()
addprocs(pargs["cpus"])


function read2array(ifname)
   a=Array{Float32}[]
   geneDic=Dict()
   ifile=open(ifname,"r")
   genes=readline(ifile)
   genes=split(strip(genes),"\t")
   [geneDic[i]=genes[i] for i in collect(1:length(genes))]
   for line in eachline(ifile)
        line=split(strip(line),"\t")
        x=[parse(Float32,i) for i in line]
        push!(a,x)
   end
  return(geneDic,a)
end



@everywhere using RCall
@everywhere R"(library(survival))"
@everywhere using StatsBase
@everywhere function coefRcall_1(a11)
   co_sub=Float32[0 0 0 0 0]
   for i in 1:niteration
      rand_index=sample(collect(1:size(a,1)),size(a,1),replace=false)
      g=Int(ceil(size(a,1)/split_groups))
      split_index=collect(Iterators.partition(rand_index, g)) #
      for j in 1:split_groups
           aj=a11[split_index[j]]
           clinicj=clinic[split_index[j],:]
           @rput aj
           @rput clinicj
           co=R"(summary(coxph(Surv(clinicj$time, clinicj$status) ~ aj)))"
           jco=rcopy(co)
           jcoef=jco[:coefficients]
           jint=jco[:conf_int]
           jcon=[jcoef[1] jcoef[2] jcoef[5] jint[3] jint[4]]
           co_sub=vcat(co_sub,jcon)
      end
   end
   co_sub=co_sub[2:end,:]
   co_sub=mean(co_sub,dims=(1))
   return co_sub
end


@everywhere function subSet(sub_range)
   sleep(30)
   outfile = open(string(outfilename,"_",string(sub_range[1]),"_",string(sub_range[end]),".txt"), "w")
   sub_dic=Dict()
   for inode in sub_range #run i node
      icoef=coefRcall_1(a[:,inode])
      println(outfile,inode,"\t",icoef[1],"\t",icoef[2],"\t",icoef[3],"\t",icoef[4],"\t",icoef[5])
      sub_dic[inode]=icoef
   end
   close(outfile)
   return sub_dic
end


@everywhere using DelimitedFiles
function write2DelimitedFiles(results)
   aa = Array[]
   push!(aa,["id","coef","exp","pvalue","lower95","upper95"])
   [push!(aa, [gdic[inode],v[1],v[2],v[3],v[4],v[5]]) for iDic in results for (inode,v) in iDic]
   outfinalfile = open(string(outfilename,"_final.survival",".txt"), "w")
   writedlm(outfinalfile,aa)
   close(outfinalfile)
end

function writekeys(d)
   ofile = open("allkeys_RNAseq.txt", "w")
   for (k,v) in d
      println(ofile,k,"\t",v)
   end
end


function main()

   println("In running, it might take a while!")
   n=Int(ceil(size(a,2)/cpus))
   x=collect(1:size(a,2))
   run_range=[x[i:min(i+n-1,length(x))] for i in 1:n:length(x)]
   results=pmap((sub_range)->subSet(sub_range),[sub_range for sub_range in run_range])
   write2DelimitedFiles(results)
end



cpus=Int(floor(pargs["cpus"]))
@eval @everywhere cpus=$cpus


outfilename=pargs["o"]
@everywhere outfilename= $outfilename

ifname=pargs["i"]
println(ifname)

gdic,a=read2array(ifname)
writekeys(gdic)
#gdic=dicKeys(gdic)
#@eval @everywhere gdic=$gdic

a=hcat(a...)
a=transpose(a)
a = SharedArray{Float32,2}(a[1:end,1:end])
@eval @everywhere a=$a
println("your data, first 2 rows and columns")
println(a[1:2,1:2])

@everywhere using CSV
clinicifname=pargs["clinic"]
clinic=CSV.read(clinicifname,delim="\t")
@eval @everywhere clinic=$clinic


split_groups =pargs["subsampling"]
if size(a,1) < 100
  split_groups =1
end  
@eval @everywhere split_groups = $split_groups
percent_cutoff=pargs["percentcutoff"]
@eval @everywhere percent_cutoff = $percent_cutoff
niteration =pargs["iteration"]
@eval @everywhere niteration = $niteration

main()

